@@ -4,6 +4,7 @@ use std::path;
 use lazy_static::lazy_static;
 use serde::{Deserialize, Deserializer, Serialize, Serializer};
 
+use crate::archive::FlatpakArchiveType;
 use crate::format::FlatpakManifestFormat;
 
 pub const ARCHIVE: &str = "archive";
@@ -18,32 +19,28 @@ pub const PATCH: &str = "patch";
 pub const EXTRA_DATA: &str = "extra-data";
 
 lazy_static! {
-    pub static ref SOURCE_TYPES: Vec<String> = vec![
-        ARCHIVE.to_string(),
-        GIT.to_string(),
-        BAZAAR.to_string(),
-        SVN.to_string(),
-        DIR.to_string(),
-        FILE.to_string(),
-        SCRIPT.to_string(),
-        SHELL.to_string(),
-        PATCH.to_string(),
-        EXTRA_DATA.to_string(),
+    pub static ref CODE_TYPES: Vec<FlatpakSourceType> = vec![
+        FlatpakSourceType::Archive,
+        FlatpakSourceType::Git,
+        FlatpakSourceType::Bazaar,
+        FlatpakSourceType::Svn,
+        FlatpakSourceType::Dir,
     ];
-    pub static ref CODE_TYPES: Vec<String> = vec![
-        ARCHIVE.to_string(),
-        GIT.to_string(),
-        BAZAAR.to_string(),
-        SVN.to_string(),
-        DIR.to_string(),
+    pub static ref VCS_TYPES: Vec<FlatpakSourceType> = vec![
+        FlatpakSourceType::Git,
+        FlatpakSourceType::Bazaar,
+        FlatpakSourceType::Svn,
     ];
-    pub static ref VCS_TYPES: Vec<String> = vec![GIT.to_string(), BAZAAR.to_string(), SVN.to_string(),];
 }
 
 #[derive(Clone)]
 #[derive(Deserialize)]
 #[derive(Debug)]
 #[derive(Hash)]
+#[derive(PartialEq)]
+/// The Flatpak sources can be of multiple different types, determined
+/// by the `type` field. The type of the Flatpak source will determine which
+/// other fields should be populated.
 pub enum FlatpakSourceType {
     Archive,
     Git,
@@ -62,6 +59,31 @@ impl Default for FlatpakSourceType {
     }
 }
 impl FlatpakSourceType {
+    /// Determines if a Flatpak source points to a code project.
+    /// See [struct@crate::source::CODE_TYPES] for the list of code types.
+    pub fn is_code(&self) -> bool {
+        CODE_TYPES.contains(self)
+    }
+
+    /// Determines if a Flatpak source points to a version-control system
+    /// repository.
+    /// See [struct@crate::source::VCS_TYPES] for the list of VCS types.
+    pub fn is_vcs(&self) -> bool {
+        VCS_TYPES.contains(self)
+    }
+
+    /// Determines if mirror urls can be used with that source type.
+    pub fn supports_mirror_urls(&self) -> bool {
+        // FIXME why are mirror urls not supported for types git, svn and bzr?
+        if *self == FlatpakSourceType::Archive {
+            return true;
+        }
+        if *self == FlatpakSourceType::File {
+            return true;
+        }
+        false
+    }
+
     pub fn to_string(&self) -> String {
         match &self {
             FlatpakSourceType::Archive => ARCHIVE.to_string(),
@@ -111,20 +133,26 @@ impl FlatpakSourceType {
     }
 }
 
-pub fn serialize_to_string<S>(x: &FlatpakSourceType, s: S) -> Result<S::Ok, S::Error>
+pub fn serialize_to_string<S>(x: &Option<FlatpakSourceType>, s: S) -> Result<S::Ok, S::Error>
 where
     S: Serializer,
 {
-    s.serialize_str(&x.to_string())
+    if let Some(build_system) = x {
+        return s.serialize_str(&build_system.to_string());
+    }
+    panic!("This should not happen.");
 }
 
-pub fn deserialize_from_string<'de, D>(deserializer: D) -> Result<FlatpakSourceType, D::Error>
+pub fn deserialize_from_string<'de, D>(deserializer: D) -> Result<Option<FlatpakSourceType>, D::Error>
 where
     D: Deserializer<'de>,
 {
     let buf = String::deserialize(deserializer)?;
 
-    FlatpakSourceType::from_string(&buf).map_err(serde::de::Error::custom)
+    match FlatpakSourceType::from_string(&buf) {
+        Ok(b) => Ok(Some(b)),
+        Err(e) => Err(e).map_err(serde::de::Error::custom),
+    }
 }
 
 #[derive(Clone)]
@@ -145,111 +173,6 @@ pub enum FlatpakSourceItem {
     Path(String),
     Description(FlatpakSource),
 }
-impl FlatpakSourceItem {
-    pub fn get_url(&self) -> Option<String> {
-        let source_description = match self {
-            FlatpakSourceItem::Path(_) => return None,
-            FlatpakSourceItem::Description(sd) => sd,
-        };
-        match &source_description.url {
-            Some(s) => Some(s.to_string()),
-            None => None,
-        }
-    }
-
-    pub fn get_all_mirror_urls(&self) -> Vec<String> {
-        let mut response: Vec<String> = vec![];
-
-        let source_description = match self {
-            FlatpakSourceItem::Path(_) => return response,
-            FlatpakSourceItem::Description(sd) => sd,
-        };
-        if let Some(urls) = &source_description.mirror_urls {
-            for url in urls {
-                response.push(url.to_string());
-            }
-        }
-        return response;
-    }
-
-    pub fn get_all_urls(&self) -> Vec<String> {
-        let mut response: Vec<String> = vec![];
-        let source_description = match self {
-            FlatpakSourceItem::Path(_) => return response,
-            FlatpakSourceItem::Description(sd) => sd,
-        };
-        if let Some(url) = &source_description.url {
-            response.push(url.to_string());
-        }
-        if let Some(urls) = &source_description.mirror_urls {
-            for url in urls {
-                response.push(url.to_string());
-            }
-        }
-        return response;
-    }
-
-    pub fn get_type_name(&self) -> String {
-        return match self {
-            FlatpakSourceItem::Path(_) => "path".to_string(),
-            FlatpakSourceItem::Description(d) => {
-                if let Some(t) = &d.r#type {
-                    return t.to_string();
-                }
-                return "empty".to_string();
-            }
-        };
-    }
-
-    pub fn has_commit(&self) -> bool {
-        return match self {
-            FlatpakSourceItem::Path(_) => false,
-            FlatpakSourceItem::Description(d) => d.commit.is_some(),
-        };
-    }
-
-    pub fn has_tag(&self) -> bool {
-        return match self {
-            FlatpakSourceItem::Path(_) => false,
-            FlatpakSourceItem::Description(d) => d.tag.is_some(),
-        };
-    }
-
-    pub fn has_branch(&self) -> bool {
-        return match self {
-            FlatpakSourceItem::Path(_) => false,
-            FlatpakSourceItem::Description(d) => d.branch.is_some(),
-        };
-    }
-
-    pub fn type_is_valid(&self) -> bool {
-        return match self {
-            FlatpakSourceItem::Path(_) => true,
-            FlatpakSourceItem::Description(d) => {
-                if let Some(t) = &d.r#type {
-                    return SOURCE_TYPES.contains(&t);
-                }
-                return false;
-            }
-        };
-    }
-
-    pub fn type_is_empty(&self) -> bool {
-        return match self {
-            FlatpakSourceItem::Path(_) => false,
-            FlatpakSourceItem::Description(d) => d.r#type.is_none(),
-        };
-    }
-
-    pub fn supports_mirror_urls(&self) -> bool {
-        let type_name = self.get_type_name();
-        // FIXME why are mirror urls not supported for types git, svn and bzr.
-        if type_name == ARCHIVE || type_name == FILE {
-            return true;
-        }
-        return false;
-    }
-}
 
 #[derive(Clone)]
 #[derive(Deserialize)]
@@ -263,11 +186,11 @@ impl FlatpakSourceItem {
 /// distinguished by the type property.
 pub struct FlatpakSource {
     /// Defines the type of the source description.
-    /// It is not explicit in the flatpak-manifest man page,
-    /// but we only found 1 source in all our dataset with an empty
-    /// type, so we assume that the field is actually required.
+    #[serde(deserialize_with = "crate::source::deserialize_from_string")]
+    #[serde(serialize_with = "crate::source::serialize_to_string")]
     #[serde(skip_serializing_if = "Option::is_none")]
-    pub r#type: Option<String>,
+    #[serde(default)]
+    pub r#type: Option<FlatpakSourceType>,
 
     /// An array of shell commands.
     /// types: script, shell
@@ -342,21 +265,12 @@ pub struct FlatpakSource {
     pub branch: Option<String>,
 
     /// The type of archive if it cannot be guessed from the path.
-    /// Possible values are:
-    ///   * "rpm",
-    ///   * "tar",
-    ///   * "tar-gzip",
-    ///   * "tar-compress",
-    ///   * "tar-bzip2",
-    ///   * "tar-lzip",
-    ///   * "tar-lzma",
-    ///   * "tar-lzop",
-    ///   * "tar-xz",
-    ///   * "zip",
-    ///   * "7z",
     /// types: archive
+    #[serde(deserialize_with = "crate::archive::deserialize_from_string")]
+    #[serde(serialize_with = "crate::archive::serialize_to_string")]
     #[serde(skip_serializing_if = "Option::is_none")]
-    pub archive_type: Option<String>,
+    #[serde(default)]
+    pub archive_type: Option<FlatpakArchiveType>,
 
     /// The commit to use from the git repository.
     /// If branch is also specified, then it is verified that the branch/tag is at this specific commit.
@@ -446,11 +360,8 @@ pub struct FlatpakSource {
 }
 impl FlatpakSource {
     /// Get the type for the Flatpak source.
-    pub fn get_type(&self) -> Option<String> {
-        if let Some(t) = &self.r#type {
-            return Some(t.clone());
-        }
-        None
+    pub fn get_type(&self) -> Option<FlatpakSourceType> {
+        self.r#type.clone()
     }
 
     pub fn file_path_matches(path: &str) -> bool {
@@ -494,7 +405,7 @@ impl FlatpakSource {
     }
 
     pub fn parse(format: FlatpakManifestFormat, manifest_content: &str) -> Result<FlatpakSource, String> {
-        let mut flatpak_source: FlatpakSource = match &format {
+        let flatpak_source: FlatpakSource = match &format {
             FlatpakManifestFormat::YAML => match serde_yaml::from_str(&manifest_content) {
                 Ok(m) => m,
                 Err(e) => {
@@ -522,7 +433,7 @@ impl FlatpakSource {
         format: FlatpakManifestFormat,
         manifest_content: &str,
     ) -> Result<Vec<FlatpakSource>, String> {
-        let mut flatpak_sources: Vec<FlatpakSource> = match &format {
+        let flatpak_sources: Vec<FlatpakSource> = match &format {
             FlatpakManifestFormat::YAML => match serde_yaml::from_str(&manifest_content) {
                 Ok(m) => m,
                 Err(e) => {
@@ -549,13 +460,55 @@ impl FlatpakSource {
     }
 
     pub fn is_valid(&self) -> Result<(), String> {
-        if let Some(source_type) = &self.r#type {
-            if !SOURCE_TYPES.contains(&source_type) {
-                return Err(format!("Invalid source type {}.", source_type));
-            }
+        if self.url.is_none() && self.path.is_none() && self.commands.is_none() {
+            return Err("There should be at least a url, a path or inline commands in a source!".to_string());
         }
         Ok(())
     }
+
+    pub fn get_url(&self) -> Option<String> {
+        match &self.url {
+            Some(s) => Some(s.to_string()),
+            None => None,
+        }
+    }
+
+    pub fn get_all_mirror_urls(&self) -> Vec<String> {
+        let mut response: Vec<String> = vec![];
+        if let Some(urls) = &self.mirror_urls {
+            for url in urls {
+                response.push(url.to_string());
+            }
+        }
+        return response;
+    }
+
+    pub fn get_all_urls(&self) -> Vec<String> {
+        let mut response: Vec<String> = vec![];
+        if let Some(url) = &self.url {
+            response.push(url.to_string());
+        }
+        if let Some(urls) = &self.mirror_urls {
+            for url in urls {
+                response.push(url.to_string());
+            }
+        }
+        return response;
+    }
+
+    pub fn get_type_name(&self) -> String {
+        if let Some(t) = self.get_type() {
+            return t.to_string();
+        }
+        return "empty".to_string();
+    }
+
+    pub fn supports_mirror_urls(&self) -> bool {
+        if let Some(t) = self.get_type() {
+            return t.supports_mirror_urls();
+        }
+        return false;
+    }
 }
 
 #[derive(Clone)]
@@ -598,6 +551,7 @@ mod tests {
             Err(e) => std::panic::panic_any(e),
             Ok(source) => {
                 assert_eq!(source.path, Some("apply_extra.sh".to_string()));
+                assert_eq!(source.get_type(), Some(FlatpakSourceType::File));
             }
         }
     }
@@ -641,7 +595,7 @@ mod tests {
     }
 
     #[test]
-    pub fn test_parse_invalid_source_type() {
+    pub fn test_parse_invalid_type() {
         let source_manifest = r###"
             type: not_a_valid_source_type
             path: apply_extra.sh
@@ -656,6 +610,45 @@ mod tests {
         }
     }
 
+    #[test]
+    pub fn test_parse_archive_type() {
+        let source_manifest = r###"
+            type: archive
+            url: https://ftp.gnu.org/gnu/glibc/glibc-2.0.1.tar.gz
+            archive_type: tar-gz
+        "###;
+        match FlatpakSource::parse(FlatpakManifestFormat::YAML, source_manifest) {
+            Ok(source) => {
+                assert!(source.url.is_some());
+                assert_eq!(source.get_type(), Some(FlatpakSourceType::Archive));
+            }
+            Err(e) => {
+                panic!(
+                    "We should be able to parse a source manifest with an archive type: {}",
+                    e
+                );
+            }
+        }
+    }
+
+    #[test]
+    pub fn test_parse_invalid_archive_type() {
+        let source_manifest = r###"
+            type: archive
+            archive-type: blahblah
+            url: https://ftp.gnu.org/gnu/glibc/glibc-2.0.1.tar.gz
+        "###;
+        match FlatpakSource::parse(FlatpakManifestFormat::YAML, source_manifest) {
+            Ok(_source) => {
+                println!("{:?}", _source);
+                panic!("We should not be able to parse a source manifest with an invalid source type");
+            }
+            Err(e) => {
+                assert!(e.to_string().contains("Invalid archive type"));
+            }
+        }
+    }
+
     #[test]
     pub fn test_parse_missing_source_type() {
         let source_manifest = r###"
@@ -665,6 +658,7 @@ mod tests {
         match FlatpakSource::parse(FlatpakManifestFormat::YAML, source_manifest) {
             Ok(source) => {
                 assert!(source.url.is_some());
+                assert!(source.get_type().is_none());
             }
             Err(e) => {
                 panic!(
@@ -674,4 +668,16 @@ mod tests {
             }
         }
     }
+
+    #[test]
+    pub fn test_parse_random_yaml_file() {
+        let source_manifest = r###"
+            title: "Copying the Diagram to the Clipboard"
+            content: []
+            description: "This is where you click for the action to happen"
+        "###;
+        if FlatpakSource::parse(FlatpakManifestFormat::YAML, source_manifest).is_ok() {
+            panic!("We should not parse a random yaml file as a source manifest",);
+        }
+    }
 }
